use io;

function input() {
    local usage = "Usage: localsolver order_picking.lsp inFileName=instanceFile"
            + " [outFileName=outputFile] [lsTimeLimit=timeLimit]";
    if (inFileName == nil) throw usage;
    local inFile = io.openRead(inFileName);
    nbOrders = inFile.readInt() + 1;
    for [i in 0...nbOrders][j in 0...nbOrders]
        distances[i][j] = inFile.readInt();
    inFile.close();
}

function model() {
    // Declare the list containing the picking order
    pickingList <- list(nbOrders);

    // All orders must be picked
    constraint count(pickingList) == nbOrders;

    // The objective is to minimize the total distance required to pick 
    // all the orders and to go back to the initial position
    objective <- sum(0...nbOrders - 1, i => distances[pickingList[i]][pickingList[i + 1]]) 
        + distances[pickingList[nbOrders - 1]][pickingList[0]];
    
    // Store the index of the initial position in the list.
    // It will be used at the end to write the solution starting from the initial point.
    indexInitialPosition <- indexOf(pickingList, 0);

    minimize objective;
}

function param() {
    if (lsTimeLimit == nil) lsTimeLimit = 10;
}

function output() {
    if (outFileName != nil) {
        outFile = io.openWrite(outFileName);
        outFile.println(objective.value);
        println("Solution written in file ", outFileName);
        for [i in 0...nbOrders] {
            local index = (indexInitialPosition.value + i) % nbOrders;
            outFile.print(pickingList.value[index], " ");
        }
    }
}
